<?php
/******************************************************************************/
//                                                                            //
//                           InstantCMS v1.10.6                               //
//                        http://www.instantcms.ru/                           //
//                                                                            //
//                   written by InstantCMS Team, 2007-2015                    //
//                produced by InstantSoft, (www.instantsoft.ru)               //
//                                                                            //
//                        LICENSED BY GNU/GPL v2                              //
//                                                                            //
/******************************************************************************/

class p_auth_logs extends cmsPlugin {

    public $config = array (
        'P_STORE_COUNT' => 10,
        'PL_SHOW_USER_TAB' => 1
    );

    public function __construct(){

        $this->info = array(
            'plugin'      => 'p_auth_logs',
            'title'       => 'Логирование авторизаций',
            'description' => 'Логирование удачных и неудачных авторизаций',
            'author'      => '<a href="http://www.instantvideo.ru/" target="_blank">InstantMedia team</a>',
            'version'     => '1.2',
            'published'   => 1
        );

        $this->events = array(
            'SIGNIN_USER',
            'USER_LOGIN',
            'USER_PROFILE',
            'GET_ROUTE_USERS',
            'GET_USERS_ACTION_GET_AUTH_LOGS'
        );

        parent::__construct();

    }

    public function install(){

        if (!$this->inDB->isTableExists('cms_auth_logs')){

            $this->inDB->query("CREATE TABLE `cms_auth_logs` (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `is_success` tinyint(1) NOT NULL DEFAULT '1',
                `ip` int(32) unsigned NOT NULL,
                `user_id` int(11) NOT NULL,
                `auth_date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                `useragent` varchar(250) NOT NULL DEFAULT '',
                PRIMARY KEY (`id`),
                KEY `user_id` (`user_id`,`auth_date`)
              ) ENGINE=MyISAM  DEFAULT CHARSET=utf8;");

        }

        return parent::install();

    }

    /**
     * Обработка событий
     * @param string $event
     * @param array $auth_data
     * @return html
     */
    public function execute($event, $auth_data){

        switch ($event){
            case 'SIGNIN_USER': return $this->logSignin($auth_data);
            case 'USER_LOGIN': return $this->log($auth_data);
            case 'USER_PROFILE': return $this->getTab($auth_data);
            case 'GET_ROUTE_USERS': return $this->eventGetRoutes($auth_data);
            case 'GET_USERS_ACTION_GET_AUTH_LOGS': return $this->getLogs();
        }

        return $auth_data;

    }

    private function deleteOldAuth($user_id) {
        $this->inDB->query("DELETE FROM `cms_auth_logs` WHERE id IN (SELECT id FROM (SELECT id FROM `cms_auth_logs` WHERE user_id = '{$user_id}' ORDER BY `id` DESC LIMIT {$this->config['P_STORE_COUNT']}, 10) as x)");
    }


    private function logSignin($user) {

        $user_id = $this->inDB->get_field('cms_users', "login = '{$user['login']}'", 'id');
        if(!$user_id){ return false; }

        $this->inDB->insert('cms_auth_logs', array(
            'ip'=>sprintf('%u', ip2long($_SERVER['REMOTE_ADDR'])),
            'user_id'=>$user_id,
            'is_success'=>0,
            'useragent'=>cmsCore::strClear((string)$_SERVER['HTTP_USER_AGENT']),
        ));

        $this->deleteOldAuth($user_id);

        return false;

    }

    private function log($user) {

        $this->inDB->insert('cms_auth_logs', array(
            'ip'=>sprintf('%u', ip2long($_SERVER['REMOTE_ADDR'])),
            'user_id'=>$user['id'],
            'useragent'=>cmsCore::strClear((string)$_SERVER['HTTP_USER_AGENT']),
        ));

        $this->deleteOldAuth($user['id']);

        return $user;

    }

    private function getTab($user){

        $inUser = cmsUser::getInstance();

        if(!$inUser->id){
            return false;
        }

        if(!$this->config['PL_SHOW_USER_TAB'] && !$inUser->is_admin){
            return false;
        }

        if(!$inUser->is_admin && ($inUser->id != $user['id'])){
            return false;
        }

        global $_LANG;

        $this->info['tab']       = $_LANG['PLOG_TAB_TITLE'];
        $this->info['ajax_link'] = '/users/get_auth_logs?user_id=' . $user['id'];

        return '';

    }

    private function eventGetRoutes($routes) {

		$add_routes[] = array(
					'_uri'  => '/^users\/get_auth_logs$/i',
					'do'    => 'get_auth_logs'
				 );

		foreach($add_routes as $route){
			array_unshift($routes, $route);
		}

        return $routes;

    }

    private function getLogs() {

        $inUser = cmsUser::getInstance();

        if(!cmsCore::isAjax() || !$inUser->id){
            cmsCore::error404();
        }

        if(!$this->config['PL_SHOW_USER_TAB'] && !$inUser->is_admin){
            cmsCore::error404();
        }

        $user = cmsUser::getShortUserData(cmsCore::request('user_id', 'int', $inUser->id));
        if(!$user){ cmsCore::error404(); }

        if(!$inUser->is_admin && ($inUser->id != $user['id'])){
            cmsCore::error404();
        }

        $logs = $this->inDB->get_table('cms_auth_logs', "user_id = '{$user['id']}' ORDER BY auth_date DESC");
        if($logs){

            cmsCore::includeFile('plugins/p_auth_logs/phpUserAgent.php');
            cmsCore::includeFile('plugins/p_auth_logs/phpUserAgentStringParser.php');

            foreach ($logs as $key => $log) {
                $ua = new phpUserAgent($log['useragent']);
                $logs[$key]['browser']    = ucfirst($ua->getBrowserName()).' '.$ua->getBrowserVersion();
                $logs[$key]['fauth_date'] = cmsCore::dateFormat($log['auth_date']);
                $logs[$key]['fip']        = long2ip($log['ip']);
                unset($ua);
            }
        }

        ob_start();

        cmsPage::initTemplate('plugins', 'p_auth_logs')->
                assign('logs', $logs)->
                display('p_auth_logs.tpl');

        cmsCore::halt(ob_get_clean());

    }

}
